/*****************************************************************************
 *
 * cgi.c - Common functions for cgi functions.
 *
 * Copyright (c) 2004 Ben Clewett (bclewett@perfparse.org)
 * Last Modified:   2004-05-28
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <libpp_common.h>
#include "cgi.h"
#include <gettext.h>

int icgi(char *sName, int iDefault) 
{ 
	char *s = scgi(sName);
	if (s[0]) return atoi(s);
	return iDefault;
}
char *scgi(char *sName)
{
	static char *sReturn = NULL;
	static char *sQuery;
	char *sFound, s, *sStart, *sEnd;
	int iVarLen, i, iIn, iOut = 0;
	int e0, e1;

	if (sReturn != NULL) free(sReturn);
	sReturn = NULL;
	sQuery = getenv("QUERY_STRING");
	if (!sQuery) return "";

	while (1) {

		/* First Hit */
		sFound = (char *)strstr(sQuery, sName);
		if (!sFound) {
			return "";
		}

		/* Correct hit? */
		sStart = sFound - 1;
		if (sStart[0] != '?' && sStart[0] != '&' && sStart + 1 != sQuery) {
			/* subset of another variable, or a value */
			/* Look for next hit: */
			sQuery = sFound + 1;
			continue;
		}
		sEnd = sFound + strlen(sName);
		if (sEnd[0] != '=') {
			/* subset of another variable, or a value */
			/* Look for next hit: */
			sQuery = sFound + 1;
			continue;
		}

		break;

	}

	/* Count varaible: */
	sStart = sEnd + 1;
	iVarLen = 0;
	while (1) {
		s = sStart[iVarLen++];
		if (s == '&' || s == 0) break;
	}

	/* Copy in, unescaping as we go: */
	sReturn = (char *)malloc(iVarLen + 1);
	iOut = 0;
	for (iIn = 0; iIn < iVarLen - 1; iIn++) {
		if (sStart[iIn] == '%') {
			e0 = sStart[iIn + 1];
			e1 = sStart[iIn + 2];
			if (e0 >= '0' && e0 <= '9') {
				e0 -= '0';
			} else if (e0 >= 'A' && e0 <= 'F') {
				e0 += 10 - 'A';
			} else if (e0 >= 'a' && e0 <= 'f') {
				e0 += 10 - 'a';
			} else e0 = 0;
			if (e1 >= '0' && e1 <= '9') {
				e1 -= '0';
			} else if (e1 >= 'A' && e1 <= 'F') {
				e1 += 10 - 'A';
			} else if (e1 >= 'a' && e1 <= 'f') {
				e1 += 10 - 'a';
			} else e1 = 0;
			i = e0 * 16 + e1;
			sReturn[iOut++] = i;
			iIn += 2;
		} else if (sStart[iIn] == '+') {
			sReturn[iOut++] = ' ';
		} else {
			sReturn[iOut++] = sStart[iIn];
		}
	}
	sReturn[iOut] = 0;

	return sReturn;

}

/* [TODO], Handle buffer overflow, or define a dynamic buffer */
char *url_encode(char *sIn)
{

	int a, b;
	char c;
	static char sOut[256];

	for (a = b = 0; b < 251 && sIn[a]; b++) {
		c = sIn[a++];
		if (c == ' ') {
			sOut[b] = '+';
		} else if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c >= '0' && c <= '9')) {
			sOut[b] = c;
		} else if (c == ' ') {
			sOut[b] = '+';
		} else {
			sOut[b] = 0;
			sprintf(sOut, "%s%%%02X", sOut, c);
			b += 2;
		}
	}

	sOut[b] = 0;

	return sOut;

}


void getCGIDBVars()
{

	char *s;

	/* [TODO] No buffer overflow check on incomming CGI. */

	if ((s = scgi("db_user"))[0] > ' ') {
		config_set_value_as_string(CONFIG_ITEM_ID_DB_USER, s);
	}

	if ((s = scgi("db_pass"))[0] > ' ') {
		config_set_value_as_string(CONFIG_ITEM_ID_DB_PWD, s);
	}

	if ((s = scgi("db_name"))[0] > ' ') {
		config_set_value_as_string(CONFIG_ITEM_ID_DB_DATABASE, s);
	}

	if ((s = scgi("db_host"))[0] > ' ') {
		config_set_value_as_string(CONFIG_ITEM_ID_DB_HOST, s);
	}

}



