/*****************************************************************************
 *
 * cgi_raw_report - Display report of raw data.
 *
 * Copyright (c) 2004 Ben Clewett (bclewett@perfparse.org)
 * Last Modified:   2004-05-28
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <config.h>
#include <gettext.h>
#include "libpp_common.h"
#include "cgi_raw_report.h"
#include "object_users.h"

char *getTime(int iTime);
void getRawCGI();



/* CGI  Variables */
char *sHost;
int iService;
char *sService;
int iType;
char sTimeRange[2], *sRelativeDays, *sRelativeTime;
char sFrom[22], sTo[22];



/*
 *
 *  History of a specific metric
 *
 */
void displayRawHistory()
{

	/* Accounting */
	int iStates[4] = { 0, 0, 0, 0 };
	int iStatesTime[4] = { 0, 0, 0, 0 };
	int iTotalTime = 0;

	/* Others */
	int  iState, iCount = 0, i = 0, iDisplayCount = 0, iPrevState = -1, bHeader = FALSE;
	char *s;
	char sColour[7] = "FFFFFF", *sState;
	int iFirst = 0, iLast = 0, iTime = 0, iOldTime = -1;
	char sLastRaw[255], sLastColour[7], sFirst[30], sLast[30];
	char sExtraOptions[255];
	GString		*sCGI = NULL;

	#ifdef USE_DB_POSTGRESQL
	int numberOfRows;
	#endif

	sCGI = g_string_sized_new(SQL_SIZE);

	getRawCGI();

	sprintf(sExtraOptions, "<a href=\"%s?all_raw=1\">%s</a>\n", sProgName, _("Return Raw Report."));

	showHeader(_("Raw History Report"), sExtraOptions);

	printf("<p align=center>\n");
	printf("<table width=600 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>\n");
	printf("  <table width=\"100%%\" border=0 cellpadding=0 cellspacing=0>\n");

	printf("<form method=get name=\"graph\">\n");
	printf("<input type=hidden name=\"host\" value=\"%s\">\n", sHost);
	printf("<input type=hidden name=\"service\" value=\"%d\">\n", iService);
	printf("<input type=hidden name=\"raw_history\" value=\"1\">\n");

	printf("</table>\n");
	printf("<table width=\"100%%\" border=0 cellpadding=2 cellspacing=0>\n");

	/* Current Service */
	printf("  <tr>\n");
	printf("    <td align=right><font face=\"Arial, Helvetica\" size=2>\n");
	printf(_("<b>Host:</b>\n"));
	printf("    </td>\n");
	printf("    <td><font face=\"Arial, Helvetica\" size=2>\n");
	printf("%s\n", sHost);
	printf("    </td>\n");
	printf("    <td align=right><font face=\"Arial, Helvetica\" size=2>\n");
	printf(_("<b>Service:</b>\n"));
	printf("    </td>\n");
	printf("    <td><font face=\"Arial, Helvetica\" size=2>\n");
	printf("%s\n", sService);
	printf("    </td>\n");
	printf("    <td align=right><font face=\"Arial, Helvetica\" size=2>\n");
	printf(_("<b>Report:</b>")); 
	printf("    </td>\n");
	printf("    <td><font face=\"Arial, Helvetica\" size=2>\n");
	printf("<select name=\"type\">");
	printf("  <option value=\"0\"%s>%s\n", (iType == 0) ? " selected" : "", _("All events"));
	printf("  <option value=\"1\"%s>%s\n", (iType == 1) ? " selected" : "", _("Change in Events"));
	printf("  <option value=\"2\"%s>%s\n", (iType == 2) ? " selected" : "", _("Summary Only"));
	printf("</select>\n");
	printf("    </td>\n");
	printf("  </tr>\n");

	printf("<tr><td colspan=6 height=4><hr width=\"100%%\" height=4></td></tr>\n");

	/* Time Range  Type*/
	printf("<tr>\n");
	printf("  <td align=right rowspan=2><font face=\"Arial, Helvetica\" size=2><b>%s</b></font></td>\n",_("Period:"));
	printf("  <td valign=middle><font face=\"Arial, Helvetica\" size=2>");
	printf("    <input type=radio name=\"timerange\" value=\"r\" onclick=\"isAbsRelVisible()\"%s>%s",
		(sTimeRange[0] == 'r') ? " checked" : "", _("Relative"));
	printf("</td>\n");

	/* Relative Dates */
	printf("  <td align=right id=\"Rel1\"><font face=\"Arial, Helvetica\" size=2><b>%s</b></font></td>\n",_("Rel&nbsp;Period:"));
	printf("  <td valign=middle id=\"Rel2\"><font face=\"Arial, Helvetica\" size=2>");
	printf("    <input type=text name=\"relativedays\" value=\"%s\" size=2 maxlength=4>&nbsp;%s", 
			sRelativeDays, _("Days"));
	printf("  </td>\n");
	printf("  <td align=right id=\"Rel3\"><font face=\"Arial, Helvetica\" size=2>\n");
	printf("    <b>%s</b>\n",_("And&nbsp;Hours:"));
	printf("  </td>\n");
	printf("  <td id=\"Rel4\"><font face=\"Arial, Helvetica\" size=2>\n");
	printf("    <input type=text name=\"relativetime\" value=\"%s\" size=5 maxlength=5>&nbsp;%s", 
			sRelativeTime,  _("HH:MM"));
	printf("  </td>\n");
	printf("</tr>\n");


	/* Absolute Dates */
	printf("<tr>\n");
	printf("  <td><font face=\"Arial, Helvetica\" size=2>\n");
	printf("    <input type=radio name=\"timerange\" value=\"a\" onclick=\"isAbsRelVisible()\"%s>%s",
		(sTimeRange[0] == 'a') ? " checked" : "",_("Absolute"));
	printf("  </td>\n");
	printf("  <td align=right id=\"Abs1\"><font face=\"Arial, Helvetica\" size=2><b>%s</b></font></td>\n", _("Abs&nbsp;From:"));
	printf("  <td id=\"Abs2\"><input type=text name=\"from\" value=\"%s\" size=17 maxlength=20></td>\n", sFrom);
	printf("  <td align=right id=\"Abs3\"><font face=\"Arial, Helvetica\" size=2><b>%s</b></font></td>\n",_("Abs&nbsp;To:"));
	printf("  <td id=\"Abs4\"><input type=text name=\"to\" value=\"%s\" size=17 maxlength=20></td>\n", sTo);
	printf("</tr>\n");

	printf("<tr><td colspan=6 height=4><hr width=\"100%%\" height=4></td></tr>\n");


	printf("  <tr>\n");
	printf("    <td colspan=6 align=center>\n");
	printf("<input type=submit value=\"%s\">\n",_("Generate Report"));
	printf("    </td>\n");
	printf("  </tr>\n");

	/* End neatly */
	printf("</table>\n");
	printf("</td></tr></table></p>\n");

	fflush(stdout);

	iFirst = iLast = iOldTime = -1;

	/* Start the SQL */
	#ifdef USE_DB_MYSQL
	g_string_printf(s_SQL, "SELECT ctime, txt_data, nagios_status, UNIX_TIMESTAMP(ctime) FROM perfdata_service_raw WHERE host_name = '%s'", sHost);
	g_string_append_printf(s_SQL, " AND service_description = '%s'", sService);
	g_string_append_printf(s_SQL, " AND ctime BETWEEN '%s' AND '%s'", sFrom, sTo);
	g_string_append_printf(s_SQL, " %s", getDecInc("ORDER BY perfdata_service_raw.ctime"));
	#elif defined USE_DB_POSTGRESQL
	g_string_printf(s_SQL, "SELECT ctime, txt_data, nagios_status FROM perfdata_service_raw WHERE host_name = '%s'", sHost);
	g_string_append_printf(s_SQL, " AND service_description = '%s'", sService);
	g_string_append_printf(s_SQL, " AND ctime BETWEEN '%s' AND '%s'", sFrom, sTo);
	g_string_append_printf(s_SQL, " %s", getDecInc("ORDER BY perfdata_service_raw.ctime"));
	#endif
	query(s_SQL->str);

	g_string_printf(sCGI, "%s?raw_history=1&host=%s&type=%d", sProgName, url_encode(sHost), iType);	
	g_string_append_printf(sCGI, "&service=%d", iService);

	/* Main table */

	#ifdef USE_DB_MYSQL
	while ((result_row = mysql_fetch_row(query_result))) {
	#elif defined USE_DB_POSTGRESQL
	numberOfRows = rows();
	for (currentRow = 0; currentRow<numberOfRows; currentRow++){
	#endif

		if (iType != 2 && !bHeader) {

			printf("<table width=\"100%%\" border=1 cellspacing=0 cellpadding=2>\n");
			fflush(stdout);

			printf("<tr>\n");

			printf("  <td bgcolor=\"#%s\" align=center width=\"1%%\"><nobr><font face=\"Arial, Helvetica\" size=2>", "FFFFCC");
			printf("%s</td>\n",_("Status"));

			printf("  <td bgcolor=\"#%s\" align=center width=\"1%%\"><nobr><font face=\"Arial, Helvetica\" size=2>", "FFFFCC");
			displayDecInc("perfdata_service_raw.ctime", sCGI->str, "perfdata_service_raw.ctime");
			printf("%s</td>\n",_("Last&nbsp;Check"));

			printf("  <td bgcolor=\"#%s\" align=center width=\"1%%\"><nobr><font face=\"Arial, Helvetica\" size=2>", "FFFFCC");
			printf("%s</td>\n",_("&#8710;&nbsp;Time"));

			printf("  <td bgcolor=\"#%s\" align=center><font face=\"Arial, Helvetica\" size=2>", "FFFFCC");
			displayDecInc("perfdata_service_raw.txt_data", sCGI->str, "perfdata_service_raw.ctime");
			printf("%s</td>\n",_("Plugin&nbsp;Output"));

			printf("</tr>\n");

			bHeader = TRUE;
		}

		/* Time range */
		#ifdef USE_DB_MYSQL
		iTime = iData(3);
		#elif defined USE_DB_POSTGRESQL
		iTime = tData(0);
		#endif
		if (iFirst == -1) {
			iFirst = iLast = iTime;
			strcpy(sFirst, sData(0));
			strcpy(sLast, sData(0));
		} else {
			if (iTime < iFirst) {
				iFirst = iTime;
				strcpy(sFirst, sData(0));
			}
			if (iTime > iLast) {
				iLast = iTime;
				strcpy(sLast, sData(0));
			}
		}
		
		/* Event type */
		iState = iData(2);
		
		strcpy(sColour, sStateColour[iState]);
		sState = sStates[iState];

		/* Accounting */
		iStates[iState]++;
		iCount++;

		/* No display, just calc the stats */
		if (iType == 2) {
			if (iDisplayCount > 0 && iOldTime > 0) {
				iTotalTime += (iTime - iOldTime);
				iStatesTime[iPrevState] += (iTime - iOldTime);
			}
			iDisplayCount++;
			iOldTime = iTime;
		}


		/* Display */
		if (iType == 0 || (iType == 1 && iState != iPrevState)) {

			/* Display end of last row: */
			if (iDisplayCount > 0) {

				if (iOldTime > 0) {
					iTotalTime += (iTime - iOldTime);
					iStatesTime[iPrevState] += (iTime - iOldTime);
				}

				/* Time Difference */
				printf("  <td bgcolor=\"#%s\"><font face=\"Arial, Helvetica\" size=2><nobr>&nbsp;%s&nbsp;</td>\n", 
					sLastColour, (iOldTime > 0) ? getTime(iTime - iOldTime) : "");

				/* Output */
				printf("  <td bgcolor=\"#%s\"><font face=\"Arial, Helvetica\" size=2>&nbsp;", sLastColour);
				s = sLastRaw;
				for (i = 0; s[i]; i++) {
					printf("%c", s[i]);
					if (s[i] == ',') printf(" ");
				}
				printf("  &nbsp;</td>\n");
	
				printf("</tr>\n");

			}

			strcpy(sLastRaw, sData(1));
			strcpy(sLastColour, sColour);

			/* Display start of this row: */

			printf("<tr>\n");

			/* State */
			printf("  <td bgcolor=\"#%s\"><font face=\"Arial, Helvetica\" size=2>&nbsp;%s&nbsp;</td>\n", sColour, sState);

			/* Time */
			printf("  <td bgcolor=\"#%s\"><font face=\"Arial, Helvetica\" size=2><nobr>&nbsp;%s&nbsp;</td>\n", sColour, sData(0));

			iDisplayCount++;
			iOldTime = iTime;
		}

		iPrevState = iState;

	} // END while


	/* Just stats */
	if (iType == 2 && iDisplayCount) {
		if (iOldTime > 0) {
			iTotalTime += (iTime - iOldTime);
			iStatesTime[iPrevState] += (iTime - iOldTime);
		}
	}

	/* End of last row */
	if (iDisplayCount && iType != 2) {

		if (iOldTime > 0) {
			iTotalTime += (iTime - iOldTime);
			iStatesTime[iPrevState] += (iTime - iOldTime);
		}

		/* Time Difference */
		printf("  <td bgcolor=\"#%s\"><font face=\"Arial, Helvetica\" size=2><nobr>&nbsp;%s&nbsp;</td>\n", 
			sLastColour, (iOldTime > 0) ? getTime(iTime - iOldTime) : "");

		/* Output */
		printf("  <td bgcolor=\"#%s\"><font face=\"Arial, Helvetica\" size=2>&nbsp;", sLastColour);
		s = sLastRaw;
		for (i = 0; s[i]; i++) {
			printf("%c", s[i]);
			if (s[i] == ',') printf(" ");
		}
		printf("  &nbsp;</td>\n");

		printf("</tr>\n");
	}

	/* End of main table */

	printf("</table>\n");

	/* Gant chart */
	g_string_printf(sCGI, "perfgant.png?host=%s", url_encode(sHost));
	g_string_append_printf(sCGI, "&service=%s", url_encode(sService));
	g_string_append_printf(sCGI, "&from=%s", url_encode(sFrom));
	g_string_append_printf(sCGI, "&to=%s", url_encode(sTo));
	printf("<p align=\"center\"><img src=\"%s\" border=0></p>\n", sCGI->str);




	/* Accouting */

	if (iCount) {

		printf("<p align=center><table width=600 border=1 cellspacing=0 cellpadding=2 bgcolor=\"#FFFFFF\">\n");

		printf("  <tr bgcolor=\"#FFFFCC\">\n");
		printf("    <td align=center><font face=\"Arial, Helvetica\" size=2><b>%s</b></td>\n",_("State"));
		printf("    <td align=center width=\"1%%\"><font face=\"Arial, Helvetica\" size=2><b>%s</td>\n",_("Count"));
		printf("    <td align=center width=\"1%%\"><font face=\"Arial, Helvetica\" size=2><b>%s</td>\n",_("Percent"));
		printf("    <td align=center width=\"1%%\"><font face=\"Arial, Helvetica\" size=2><b>%s</td>\n",_("Time"));
		printf("  </tr>\n");

		for (iState = 0; iState < 4; iState++) {

			double dPercent;
			if (iTotalTime > 0) {
				dPercent = (double)iStatesTime[iState] / (double)iTotalTime * 100.0;
			} else {
				dPercent = 0;
			}

			printf("  <tr>\n");
			printf("    <td><font face=\"Arial, Helvetica\" size=2>%s</td>\n", sStates[iState]);
			printf("    <td align=right><font face=\"Arial, Helvetica\" size=2>%d</td>\n", iStates[iState]);
			printf("    <td align=right><font face=\"Arial, Helvetica\" size=2>%.3f%%</td>\n", dPercent);

			if (iStates[iState]) {
				iTime = iStatesTime[iState];
			} else {
				iTime = 0;
			}

			printf("    <td align=right><font face=\"Arial, Helvetica\" size=2>%s</td>\n", 
				getTime(iTime));		

			printf("  </tr>\n");
		}

		printf("  <tr bgcolor=\"#FFFFCC\">\n");
		printf("    <td align=center colspan=4><font face=\"Arial, Helvetica\" size=2>\n");

		if (iType != 2) {

			printf("<b>%s</b> %s &nbsp; <b>%s</b> %s <br> <b>%s</b> %s &nbsp; <b>%s</b>",
				_("Start:"), sFirst, _("End:"), sLast, _("Span:"), getTime(iLast - iFirst),_("Mean &#8710; Time:"));
			printf(" %s\n", getTime((iLast - iFirst) / iDisplayCount));

		} else {

			printf("<b>%s</b> %s &nbsp; <b>%s</b> %s &nbsp; <b>%s</b> %s </b>",
				_("Start:"), sFirst, _("End:"), sLast, _("Span:"), getTime(iLast - iFirst));

		}
		printf("    </td>\n");
		printf("  <tr>\n");

		printf("</table></p>\n");
	} else {
		printf("<p align=center><font face=\"Arial, Helvetic\" size=2>%s</p>\n",_("No events found."));
	}

}





void displayRawHistorySummary()
/*
 * Tim's excelent summary data.
 */
{

	/* Accounting */
	int iSumStates[4] = { 0, 0, 0, 0 };

	/* Tmp storage */
	int iCurrentStates[4] = { 0, 0, 0, 0 };

	/* Others */
	double dPercent;
	int  iSumCurrentStates = 0, iCount = 0, i = 0, bHeader = FALSE, iTotCount = 0;
	char sColour[7] = "FFFFFF";
	char sExtraOptions[255];
	GString		*sCGI = NULL;

#ifdef USE_DB_POSTGRESQL
	displayUnsupported();
	exit(0);
#endif

	getRawCGI();

	sprintf(sExtraOptions, "<a href=\"%s?all_raw=1\">%s</a>\n", sProgName, _("Return Raw Report."));

	showHeader(_("Raw Summary Report"), sExtraOptions);

	printf("<p align=center>\n");
	printf("<table width=600 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>\n");
	printf("  <table width=\"100%%\" border=0 cellpadding=0 cellspacing=0>\n");

	printf("<form method=get name=\"graph\">\n");
	printf("<input type=hidden name=\"host\" value=\"%s\">\n", sHost);
	printf("<input type=hidden name=\"service\" value=\"%d\">\n", iService);
	printf("<input type=hidden name=\"raw_history_summary\" value=\"1\">\n");

	printf("</table>\n");
	printf("<table width=\"100%%\" border=0 cellpadding=2 cellspacing=0>\n");

	/* Current Service */
	printf("  <tr>\n");
	printf("    <td align=right><font face=\"Arial, Helvetica\" size=2>\n");
	printf(_("<b>Host:</b>\n"));
	printf("    </td>\n");
	printf("    <td><font face=\"Arial, Helvetica\" size=2>\n");
	printf("%s\n", sHost);
	printf("    </td>\n");
	printf("    <td align=right><font face=\"Arial, Helvetica\" size=2>\n");
	printf(_("<b>Service:</b>\n"));
	printf("    </td>\n");
	printf("    <td><font face=\"Arial, Helvetica\" size=2>\n");
	printf("%s\n", sService);
	printf("    </td>\n");
	printf("    <td align=right><font face=\"Arial, Helvetica\" size=2>\n");
	/* printf(_("<b>Report:</b>")); */
	printf("  &nbsp;\n");
	printf("    </td>\n");
	printf("    <td><font face=\"Arial, Helvetica\" size=2>\n");
	/*
	printf("<select name=\"type\">");
	printf("  <option value=\"0\"%s>%s\n", (iType == 0) ? " selected" : "", _("Full"));
	printf("  <option value=\"2\"%s>%s\n", (iType == 2) ? " selected" : "", _("Summary Only"));
	printf("</select>\n");
	*/
	printf("     &nbsp;\n");
	printf("    </td>\n");
	printf("  </tr>\n");

	printf("<tr><td colspan=6 height=4><hr width=\"100%%\" height=4></td></tr>\n");

	/* Time Range  Type*/
	printf("<tr>\n");
	printf("  <td align=right rowspan=2><font face=\"Arial, Helvetica\" size=2><b>%s</b></font></td>\n",_("Period:"));
	printf("  <td valign=middle><font face=\"Arial, Helvetica\" size=2>");
	printf("    <input type=radio name=\"timerange\" value=\"r\" onclick=\"isAbsRelVisible()\"%s>%s",
		(sTimeRange[0] == 'r') ? " checked" : "", _("Relative"));
	printf("</td>\n");

	/* Relative Dates */
	printf("  <td align=right id=\"Rel1\"><font face=\"Arial, Helvetica\" size=2><b>%s</b></font></td>\n",_("Rel&nbsp;Period:"));
	printf("  <td valign=middle id=\"Rel2\"><font face=\"Arial, Helvetica\" size=2>");
	printf("    <input type=text name=\"relativedays\" value=\"%s\" size=2 maxlength=4>&nbsp;%s", sRelativeDays,_("Days"));
	printf("  </td>\n");
	printf("  <td align=right id=\"Rel3\"><font face=\"Arial, Helvetica\" size=2>\n");
	/*
	 * Summary by day.  No point in asking for hours.
	 * BUT javascript expects these cells to be here!
	 */ 
	/* printf("    <b>%s</b>\n",_("And&nbsp;Hours:")); */
	printf("    &nbsp;");
	printf("  </td>\n");
	printf("  <td id=\"Rel4\"><font face=\"Arial, Helvetica\" size=2>\n");
	/* printf("    <input type=text name=\"relativetime\" value=\"%s\" size=5 maxlength=5>&nbsp;%s", sRelativeTime, _("HH:MM")); */
	printf("    &nbsp;\n");
	printf("    <input type=\"hidden\" name=\"relativetime\" value=\"00:00\">\n");
	printf("  </td>\n");
	printf("</tr>\n");


	/* Absolute Dates */

	/* Default from code I used to this is to add time. */
	/* Bit of a hack, but time not important, so remove here... */
	/* [TODO] Find some considerably better date parsing functions! */
	sscanf(sFrom, "%s", sExtraOptions);
	strcpy(sFrom, sExtraOptions);
	sscanf(sTo, "%s", sExtraOptions);
	strcpy(sTo, sExtraOptions);

	printf("<tr>\n");
	printf("  <td><font face=\"Arial, Helvetica\" size=2>\n");
	printf("    <input type=radio name=\"timerange\" value=\"a\" onclick=\"isAbsRelVisible()\"%s>%s",
		(sTimeRange[0] == 'a') ? " checked" : "",_("Absolute"));
	printf("  </td>\n");
	printf("  <td align=right id=\"Abs1\"><font face=\"Arial, Helvetica\" size=2><b>%s</b></font></td>\n", _("Abs&nbsp;From:"));
	printf("  <td id=\"Abs2\"><input type=text name=\"from\" value=\"%s\" size=10 maxlength=10></td>\n", sFrom);
	printf("  <td align=right id=\"Abs3\"><font face=\"Arial, Helvetica\" size=2><b>%s</b></font></td>\n",_("Abs&nbsp;To:"));
	printf("  <td id=\"Abs4\"><input type=text name=\"to\" value=\"%s\" size=10 maxlength=10></td>\n", sTo);
	printf("</tr>\n");

	printf("<tr><td colspan=6 height=4><hr width=\"100%%\" height=4></td></tr>\n");


	printf("  <tr>\n");
	printf("    <td colspan=6 align=center>\n");
	printf("<input type=submit value=\"%s\">\n",_("Generate Report"));
	printf("    </td>\n");
	printf("  </tr>\n");

	/* End neatly */
	printf("</table>\n");
	printf("</td></tr></table></p>\n");

	fflush(stdout);

	sCGI = g_string_sized_new(SQL_SIZE);
	g_string_printf(sCGI, "%s?raw_history_summary=1&type=%d&service=%d&host=%s", sProgName, iType, iService, url_encode(sHost));	


	/* Start the SQL */

	#ifdef USE_DB_MYSQL
	g_string_printf(s_SQL, "SELECT FROM_UNIXTIME(epoch_start) as es, count, sum_uptime, sum_warntime, sum_criticaltime, sum_undeftime FROM perfdata_raw_summary_data WHERE service_id = %d AND epoch = 86400 AND epoch_start BETWEEN UNIX_TIMESTAMP('%s') AND UNIX_TIMESTAMP('%s')", 
			iService, sFrom, sTo);
	g_string_append_printf(s_SQL, " %s", getDecInc("ORDER BY es"));
	#elif defined USE_DB_POSTGRESQL
	#endif

	/* printf("<p><pre>%s</pre></p>\n", s_SQL->str); */

	query(s_SQL->str);

	/* Main table */

	#ifdef USE_DB_MYSQL
	while ((result_row = mysql_fetch_row(query_result))) {
	#elif defined USE_DB_POSTGRESQL
	numberOfRows = rows();
	for (currentRow = 0; currentRow<numberOfRows; currentRow++){
	#endif

		if (!bHeader) {

			printf("<table bgcolor=\"#FFFFFF\" width=\"100%%\" border=1 cellspacing=0 cellpadding=2>\n");

			printf("<tr bgcolor=\"#FFFFCC\">\n");

			printf("  <td width=\"10%%\" align=\"center\"><nobr>%s", FONT);
			displayDecInc("es", sCGI->str, "es");
			printf("%s</font></nobr></td>\n", _("Date"));

			printf("  <td width=\"10%%\" align=\"center\"><nobr>%s", FONT);
			displayDecInc("count", sCGI->str, "es");
			printf("%s</td>\n",_("Count"));

			printf("  <td width=\"20%%\" align=\"center\" colspan=2><nobr>%s", FONT);
			displayDecInc("sum_uptime", sCGI->str, "es");
			printf("%s</td>\n",_("OK"));

			printf("  <td width=\"20%%\" align=\"center\" colspan=2><nobr>%s", FONT);
			displayDecInc("sum_warntime", sCGI->str, "es");
			printf("%s</td>\n",_("WARN"));

			printf("  <td width=\"20%%\" align=\"center\" colspan=2><nobr>%s", FONT);
			displayDecInc("sum_criticaltime", sCGI->str, "es");
			printf("%s</td>\n",_("CRITICAL"));

			printf("  <td width=\"20%%\" align=\"center\" colspan=2><nobr>%s", FONT);
			displayDecInc("sum_undeftime", sCGI->str, "es");
			printf("%s</td>\n",_("UNKNOWN"));

			printf("</tr>\n");

			bHeader = TRUE;
		}
		
		strcpy(sColour, "CCCCCC");

		iCount = iData(1);
		iSumCurrentStates = 0;
		for (i = 0; i < 4; i++) {
			iCurrentStates[i] = iData(i + 2);
			iSumStates[i] += iCurrentStates[i];
			iSumCurrentStates += iCurrentStates[i];
		}
		iTotCount += iCount;

		printf("<tr>\n");

		/* Date */
		printf("  <td>%s&nbsp;%s&nbsp;</font></td>", FONT, sData(0));

		/* Count */
		printf("  <td align=\"right\">%s&nbsp;%d&nbsp;</font></td>", FONT, iCount);

		/* OK, WARN, CRITICAL, UNKNOWN */
		for (i = 0; i < 4; i++) {

			char sOut[20];

			if (iSumCurrentStates > 0) {
				dPercent = (double)iCurrentStates[i] / (double)iSumCurrentStates * 100.0;
			} else {
				dPercent = 0;
			}

			sprintf(sOut, "%02d:%02d:%02d", iCurrentStates[i] / 3600, (iCurrentStates[i] / 60) / 60, iCurrentStates[i] % 60);

			printf("  <td width=\"5%%\" align=\"right\">%s&nbsp;%s&nbsp;</font></td>\n", FONT, sOut);
			printf("  <td width=\"5%%\" align=\"right\">%s&nbsp;%.1f%%&nbsp;</font></td>\n", FONT, dPercent);
		}

		printf("</tr>\n");
		

	} // END while


	/* End of main table */
	
	if (bHeader) {

		/* Totals */
		iSumCurrentStates = 0;
		for (i = 0; i < 4; i++) iSumCurrentStates += iSumStates[i];

		printf("  <tr bgcolor=\"#FFFFCC\">\n");
		printf("    <td align=\"right\">%sTotal:&nbsp;</font></td>\n", FONT);
		printf("    <td align=\"right\">%s&nbsp;%d&nbsp;</font></td>\n", FONT, iTotCount);
		for (i = 0; i < 4; i++) {

			char sOut[20];

			if (iSumCurrentStates > 0) {
				dPercent = (double)iSumStates[i] / (double)iSumCurrentStates * 100.0;
			} else {
				dPercent = 0;
			}
			sprintf(sOut, "%02d:%02d:%02d", iSumStates[i] / 3600, (iSumStates[i] / 60) / 60, iSumStates[i] % 60);

			printf("  <td align=\"right\">%s&nbsp;%s&nbsp;</font></td>\n", FONT, sOut);
			printf("  <td align=\"right\">%s&nbsp;%.1f%%&nbsp;</font></td>\n", FONT, dPercent);
		}
		printf("  </tr>\n");

		/* End table */
		printf("</table>\n");

	}

	else {

		printf("<p align=center>%s%s</font></p>\n", FONT, _("No data found."));
	}


}






void displayAllRaw()
{
	char sColour[7] = "FFFFFF";
	int  iState, i;
	char *s;
	GString		*sCGI = NULL;
	char sLastHost[76] = "";
	char sLastService[76] = "";


	#ifdef USE_DB_POSTGRESQL
	int numberOfRows;
	#endif

	header_meta();

	sCGI = g_string_sized_new(SQL_SIZE);

	getRawCGI();

	/* Options CGI: */
	if (icgi("submit", 0)) {
		user.bRawShowNormal = icgi("normal", 0);
		user.bRawShowWarning = icgi("warning", 0);
		user.bRawShowCritical = icgi("critical", 0);
		user.bRawShowUnknown = icgi("unknown", 0);
		user.iRawRefreshTime = icgi("refresh", 0);
		saveUser();
	}

	/* This calls the little used refresh time header: */
	header_refresh(user.iRawRefreshTime);

	showHeader(_("Raw Nagios Plugin Report"), "");
	
	selectGroup("all_raw", TRUE);

	if (user.sGroupName[0] == 0) return;

	g_string_sprintf(sCGI, "%s?all_raw=1", sProgName);	

	g_string_printf(s_SQL, "SELECT perfdata_service.host_name, perfdata_service.service_description, '1', perfdata_service_raw.ctime, perfdata_service_raw.txt_data, nagios_status, perfdata_service.service_id"
		" FROM perfdata_service, perfdata_service_raw, perfdata_host"
		" WHERE perfdata_service_raw.id = perfdata_service.last_perfdata_raw"
		" AND perfdata_service.host_name = perfdata_host.host_name");
	if (strcmp(user.sGroupName, "**ALL**"))
		g_string_append_printf(s_SQL, " AND group_name = '%s'", sql_escape(user.sGroupName,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, " %s", getDecInc("ORDER BY perfdata_host.host_name"));
	query(s_SQL->str);

	printf("<table width=\"100%%\" border=1 cellspacing=0 cellpadding=2>\n");
	fflush(stdout);

	printf("<tr>\n");

	printf("  <td bgcolor=\"#%s\" align=center><font face=\"Arial, Helvetica\" size=2>", "FFFFCC");
	displayDecInc("perfdata_host.host_name", sCGI->str, "perfdata_host.host_name");
	printf("Host</td>\n");

	printf("  <td bgcolor=\"#%s\" align=center><font face=\"Arial, Helvetica\" size=2>", "FFFFCC");
	displayDecInc("perfdata_service.service_description", sCGI->str, "perfdata_host.host_name");
	printf("Service</td>\n");

	printf("  <td bgcolor=\"#%s\" align=center width=\"1%%\"><font face=\"Arial, Helvetica\" size=2>", "FFFFCC");
	displayDecInc("nagios_status", sCGI->str, "perfdata_host.host_name");
	printf("%s</td>\n",_("Status"));

	printf("  <td bgcolor=\"#%s\" align=center width=\"1%%\"><font face=\"Arial, Helvetica\" size=2>", "FFFFCC");
	displayDecInc("perfdata_service_raw.ctime", sCGI->str, "perfdata_host.host_name");
	printf("%s</td>\n",_("Last Check"));

	printf("  <td bgcolor=\"#%s\" align=center><font face=\"Arial, Helvetica\" size=2>", "FFFFCC");
	displayDecInc("perfdata_service_raw.txt_data", sCGI->str, "perfdata_host.host_name");
	printf("%s</td>\n",_("Plugin&nbsp;Output"));

	printf("  <td bgcolor=\"#%s\" align=center width=\"1%%\" colspan=2><font face=\"Arial, Helvetica\" size=2>", "FFFFCC");
	printf("%s</td>\n",_("History"));

	printf("</tr>\n");

 /*
  * For thos of you reading the source code, you may like to know that the date/time is now exactly
  * '2005-01-01 00:00:00 GMT'.
  *
  * Ben Clewett.
  *
  */

	#ifdef USE_DB_MYSQL
	while ((result_row = mysql_fetch_row(query_result))) {
	#elif defined USE_DB_POSTGRESQL
	numberOfRows = rows();
	for (currentRow = 0; currentRow<numberOfRows; currentRow++){
	#endif

		iState = iData(5);

		if (iState == 0 && !user.bRawShowNormal) continue;
		if (iState == 1 && !user.bRawShowWarning) continue;
		if (iState == 2 && !user.bRawShowCritical) continue;
		if (iState == 3 && !user.bRawShowUnknown) continue;

		strcpy(sColour, sStateColour[iState]);

		printf("<tr>\n");

		/* Host */
		if (!strcmp(sData(0), sLastHost)) {
			printf("  <td bgcolor=\"#%s\"><font size=2>&nbsp;</td>\n", sColour);
		} else {
			strcpy(sLastHost, sData(0));
			sLastService[0] = 0;
			printf("  <td bgcolor=\"#%s\">%s<nobr>&nbsp;%s&nbsp;</nobr></font></td>\n", 
				sColour, FONT, sLastHost);
		}

		/* Service */
		if (!strcmp(sData(1), sLastService)) {
			printf("  <td bgcolor=\"#%s\"><font size=2>&nbsp;</td>\n", sColour);
		} else {
			strcpy(sLastService, sData(1));
			printf("  <td bgcolor=\"#%s\">%s<nobr>&nbsp;%s&nbsp;</nobr></font></td>\n", 
				sColour, FONT, sLastService);
		}

		printf("  <td bgcolor=\"#%s\"><font face=\"Arial, Helvetica\" size=2>&nbsp;%s&nbsp;</td>\n", sColour, sStates[iState]);
		printf("  <td bgcolor=\"#%s\"><font face=\"Arial, Helvetica\" size=2><nobr>&nbsp;%s&nbsp;</td>\n", sColour, sData(3));
		printf("  <td bgcolor=\"#%s\"><font face=\"Arial, Helvetica\" size=2>&nbsp;", sColour);
		s = sData(4);
		for (i = 0; s[i]; i++) {
			printf("%c", s[i]);
			if (s[i] == ',') printf(" ");
		}
		printf("  &nbsp;</td>\n");
	
		/* History */
		g_string_printf(sCGI, "%s?raw_history=1&host=%s", sProgName, url_encode(sData(0)));
		g_string_append_printf(sCGI, "&service=%d", iData(6));


		printf("  <td bgcolor=\"#%s\"><font face=\"Arial, Helvetica\" size=2><a href=\"%s\">%s</a></td>\n", sColour, sCGI->str,_("All"));

		g_string_printf(sCGI, "%s?raw_history_summary=1&host=%s", sProgName, url_encode(sData(0)));
		g_string_append_printf(sCGI, "&service=%d", iData(6));

		printf("  <td bgcolor=\"#%s\"><font face=\"Arial, Helvetica\" size=2><a href=\"%s\">%s</a></td>\n", sColour, sCGI->str,_("Summary"));

		printf("</tr>\n");

	}

	printf("</table>\n");
	
	/* Options */

	printf("<form method=\"get\">\n");
	printf("<input type=\"hidden\" name=\"all_raw\" value=1>\n");
	printf("<input type=\"hidden\" name=\"dec\" value=\"%s\">\n", scgi("dec"));
	printf("<input type=\"hidden\" name=\"inc\" value=\"%s\">\n", scgi("inc"));
	printf("<input type=\"hidden\" name=\"submit\" value=\"1\">\n");

	printf("<p align=center><table width=600 border=1 cellpadding=5 cellspacing=0 bgcolor=\"#CCCCCC\"><tr><td>\n");
  	printf("  <table width=\"100%%\" border=0 cellpadding=0 cellspacing=0>\n");
	printf("    <tr><td colspan=2 align=center>%s<b>%s</b></font></td></tr>\n", FONT, _("Options"));
	printf("    <tr>\n");
	printf("      <td align=right>%s%s:</font></td>\n", FONT, _("Show States"));
	printf("      <td>%s", FONT);
	printf("      <input type=checkbox name=\"normal\" value=\"1\"%s>&nbsp;%s", 
		(user.bRawShowNormal) ? " checked" : "", _("Normal"));
	printf("      <input type=checkbox name=\"warning\" value=\"1\"%s>&nbsp;%s&nbsp;", 
		(user.bRawShowWarning) ? " checked" : "", _("Warning"));
	printf("      <input type=checkbox name=\"critical\" value=\"1\"%s>&nbsp;%s&nbsp;", 
		(user.bRawShowCritical) ? " checked" : "", _("Critial"));
	printf("      <input type=checkbox name=\"unknown\" value=\"1\"%s>&nbsp;%s&nbsp;", 
		(user.bRawShowUnknown) ? " checked" : "", _("Unknown"));
	printf("      </td>\n");
	printf("    </tr>\n");
	printf("    <tr>\n");
	printf("      <td align=right>%s%s:</font></td>\n", FONT, _("Refresh Time"));
	printf("      <td>%s\n", FONT);
	printf("      <input type=\"text\" name=\"refresh\" value=\"%d\" size=3 maxlength=3>&nbsp;%s\n",
		user.iRawRefreshTime, _("Seconds.  Zero = Dissabled."));
	printf("      </font></td>\n");
	printf("    </tr>\n");
	printf("    <tr>\n");
	printf("      <td colspan=2 align=\"center\"><input type=\"submit\" value=\"%s\"></td>\n", _("Save"));
	printf("    </tr>\n");
	printf("  </table>\n");
	printf("</font></td></tr></table></p>\n");

	fflush(stdout);
	
}




char *getTime(int iTime)
{
	static char sTime[30];

	sprintf(sTime, "%d&nbsp;days&nbsp;%02d:%02d:%02d", 
				iTime / (3600 * 24),
				(iTime % (24 * 3600)) / 3600,
				(iTime % 3600) / 60,
				iTime % 60);

	return sTime;
}
	



void getRawCGI() 
/*
 * CGI for raw data and raw data reports.
 */
 {

	sHost = strdup(scgi("host"));
	iService = icgi("service", -1);
	iType = icgi("type", 1);

	/* Get service label */
	if (iService != -1) {
		g_string_printf(s_SQL, "SELECT service_description FROM perfdata_service WHERE service_id = '%d'", iService);
		sService = strdup(query_char(s_SQL->str));
	} else {
		sService = "";
	}

	/* Time range */
 	strncpy(sTimeRange, scgi("timerange"), 1);
	sTimeRange[1] = 0;
	if (sTimeRange[0] != 'a') sTimeRange[0] = 'r';

	sRelativeDays = strdup(scgi("relativedays"));
	sRelativeTime = strdup(scgi("relativetime"));

	if (!sRelativeDays[0]) sRelativeDays = "7";
	if (!sRelativeTime[0]) sRelativeTime = "00:00";

	strcpy(sFrom, scgi("from"));
	strcpy(sTo, scgi("to"));

	/* Populate the absolute from the relative */
	if (sTimeRange[0] == 'r') {
		time_t t0, t1;
		getRelativeToAbsolte(atoi(sRelativeDays), sRelativeTime, &t0, &t1, sFrom, sTo);
	}


 }




