/*****************************************************************************
 *
 * libpp_common.h - API to the config file used by PerfParse suit.
 *
 * Copyright (c) 2004 Yves Mettier <ymettier@libertysurf.fr>
 * Last Modified:   2004-06-11
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#ifndef __LIBPP_COMMON_H__
#define __LIBPP_COMMON_H__

#include <stdio.h>
#include <time.h>
#include <config.h>

#define USER_PERMISSIONS_RO 	"ro"
#define USER_PERMISSIONS_RW 	"rw"
#define USER_PERMISSIONS_HIDE 	"hide"

#define CONFIG_FILE "perfparse.cfg"

FILE* log_file_fopen(char*name, time_t tm, int rotate);
int pp_log (char *file, int line, char *fmt, ...);
extern char *pp_log_file;
extern int pp_log_count;

/* compress_file() returns 0 when OK, 
 * -1 if pb in the original file, 
 * and something else if pb in the compressed file */
int compress_file(const char*name);
/* check_dir_for_files_to_remove_or_compress()
 * arg2 is the directory to start the traversal
 * arg3 is a pattern for files (remove files beginning with this pattern only if specified)
 * arg3 can be NULL if you accept all files
 */
int check_dir_for_files_to_remove_or_compress(int nb_days, const char*path, const char*pattern);

typedef enum {
	CONFIG_ITEM_ID_ERROR_LOG,
	CONFIG_ITEM_ID_ERROR_LOG_ROTATE,
	CONFIG_ITEM_ID_ERROR_LOG_KEEP_N_DAYS,
	CONFIG_ITEM_ID_DROP_FILE,
	CONFIG_ITEM_ID_DROP_FILE_ROTATE,
	CONFIG_ITEM_ID_DROP_FILE_KEEP_N_DAYS,
	CONFIG_ITEM_ID_DB_USER,
	CONFIG_ITEM_ID_DB_PWD,
	CONFIG_ITEM_ID_DB_DATABASE,
	CONFIG_ITEM_ID_DB_HOST,
	CONFIG_ITEM_ID_SERVER_PORT,
	CONFIG_ITEM_ID_SERVICE_LOG,
	CONFIG_ITEM_ID_SERVICE_LOG_SAVE_POSITION,
	CONFIG_ITEM_ID_SERVICE_LOG_POSITION_MARK_PATH,
	CONFIG_ITEM_ID_HISTORY_START_TM,
	CONFIG_ITEM_ID_HISTORY_END_TM,
	CONFIG_ITEM_ID_SHOW_STATUS_BAR,
	CONFIG_ITEM_ID_DO_REPORT,
	CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_POLICY,
	CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_SUMMARY,
	CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_HOST_GROUP,
	CONFIG_ITEM_ID_DAEMON_LOCK,
	CONFIG_ITEM_ID_DAEMONIZE,
	CONFIG_ITEM_ID_PERIODIC_CLEANUP,
	CONFIG_ITEM_ID_PERIODIC_CLEANUP_LOCK,
	CONFIG_ITEM_ID_PERIODIC_CLEANUP_HOUR,
	CONFIG_ITEM_ID_STORAGE_MODULES_DIR,
	CONFIG_ITEM_ID_STORAGE_MODULES_LOAD,
	CONFIG_ITEM_ID_STORAGE_MODULES_STATUS_FILE,
	CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_FILENAME,
	CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_ROTATE,
	CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_KEEP_N_DAYS,
	CONFIG_ITEM_ID_STORAGE_GNUPLOT_OUTPUT_FILE,
	CONFIG_ITEM_ID_STORAGE_SOCKET_OUTPUT_SOCKET_HOST,
	CONFIG_ITEM_ID_STORAGE_SOCKET_OUTPUT_SOCKET_PORT,
	CONFIG_ITEM_ID_DUMMY_HOSTNAME,
	CONFIG_ITEM_ID_NO_RAW_DATA,
	CONFIG_ITEM_ID_NO_BIN_DATA,
	CONFIG_ITEM_ID_DBTOOL_UPDATE,
	CONFIG_ITEM_ID_DBTOOL_INSTALL,
	CONFIG_ITEM_ID_DBTOOL_RESET_HOST_DELETE_POLICY,
	CONFIG_ITEM_ID_SHOW_CONFIG,
	CONFIG_ITEM_ID_NO_TRANSACTIONS,
	CONFIG_ITEM_ID_NB
} config_item_id_e;

typedef enum {
	TYPE_BOOLEAN,
	TYPE_STRING,
	TYPE_INT
} type_e;

typedef struct {
	char*key;
	char*value;
	char*short_description;
	char*long_description;
	char*short_option;
	char*long_option;
	int in_config_file;
	int integer;
	int boolean;
	type_e type;
} config_item_t;

typedef enum {
	CONFIG_OPTION_HELP,
	CONFIG_OPTION_VERSION,
	CONFIG_OPTION_SHOW_CONFIG,
	CONFIG_OPTION_NONE
} config_option_e;

extern config_item_t *config;
config_option_e config_parse_config(int argc, char**argv, char*header, char*footer, int*options, const char*default_error_log);
void config_print_version(FILE*fd, char*header, char*footer);
void config_print_help(FILE*fd, char*header, char*footer, int*options);
void config_show_config(FILE*fd, char*header, char*footer, int*options);
void config_set_value_as_string(config_item_id_e id, const char*str);
#define config_get_value_as_string(id) (config[id].value)
#define config_get_value_as_integer(id) (config[id].integer)
#define config_get_value_as_boolean(id) (config[id].boolean)


#ifdef USE_GLIB12
#define g_string_hash(x) (g_str_hash((x)->str))
#define g_path_get_dirname g_dirname
#define g_string_printf g_string_sprintf
#define g_string_append_printf g_string_sprintfa
#endif

#endif
