/*****************************************************************************
 *
 * log.c - API to the config file used by PerfParse suit.
 *
 * Copyright (c) 2004 Yves Mettier <ymettier@libertysurf.fr>
 * Last Modified:   2004-09-27
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <unistd.h>
#include <glib.h>
#include <config.h>
#include "libpp_common.h"
#include <gettext.h>

#define LOG_MESSAGE_LENGTH 1024

int pp_log_count = 0;
char *pp_log_file = NULL;

FILE* log_file_fopen(char*name, time_t tm, int rotate) {
	gchar *path;
	char date[] = ".YYYYMMDD";
	FILE*log_fd;
	if(!strcmp(name,"-")) return(stderr);
	if(!strcmp(name,"stderr")) return(stderr);
	if(rotate) {
		const struct tm *stm = localtime(&tm);
		strftime(date, 10, ".%Y%m%d",stm);
	}
	path = g_strdup_printf("%s%s%s",name,
			rotate?date:"",
			rotate?".log":"");

	log_fd = fopen(path,"a");

	if (pp_log_file) free(pp_log_file);
	pp_log_file = strdup(path);

	g_free(path);

	return(log_fd);
}


int pp_log (char *file, int line, char *fmt, ...)
{
	va_list args;
	char message[LOG_MESSAGE_LENGTH];
	int message_pos=0;
	char log_date[] = "YYYY/MM/DD HH:MM:SS ";
	char *p;
	char *bufstr;
	int bufint;
	long buflong;
	time_t t;
	struct tm *tm;
	FILE*fd;

	pp_log_count++;

	va_start (args, fmt);
	for (p = fmt; (p[0] != '\0') && (message_pos < LOG_MESSAGE_LENGTH); p++)
	{
		switch (p[0])
		{
			case '%':
				switch (p[1])
				{
					case '%':
						message[message_pos++] = '%';
					case 'l':
						switch(p[2]) {
							case 'd':
							case 'i':
								buflong = va_arg (args, long);
								message_pos += snprintf (message+message_pos, LOG_MESSAGE_LENGTH-message_pos, "%ld", buflong);
								break;
							default:
								message[message_pos++] = '?';
						}
						p++;
						break;
					case 'd':
					case 'i':
						bufint = va_arg (args, int);
						message_pos += snprintf (message+message_pos, LOG_MESSAGE_LENGTH-message_pos, "%d", bufint);
						break;
					case 's':
						bufstr = va_arg (args, char *);
						while ((message_pos < LOG_MESSAGE_LENGTH) && (bufstr[0]))
						{
							message[message_pos++] = bufstr[0];
							bufstr++;
						}
						break;

					default:
						message[message_pos++] = '?';
				}
				p++;

				break;
			case '\n':
				message[message_pos++] = ' ';
				break;
			default:
				message[message_pos++] = p[0];
		}

	}
	va_end (args);
	message[message_pos] = '\0';

	t = time (0);
	tm = localtime(&t);
	strftime (log_date, sizeof(log_date), _("%Y/%m/%d %H:%M:%S"), tm);

	fd = log_file_fopen(config_get_value_as_string(CONFIG_ITEM_ID_ERROR_LOG), t,config_get_value_as_boolean(CONFIG_ITEM_ID_ERROR_LOG_ROTATE));
	if(!fd) fd = stderr;
#ifdef LOG_DEBUG_INFORMATION
	fprintf(fd, "%s [%12s:%-5d %-6ld] %s\n", log_date, file, line, (long)getpid(), message);
#else
	fprintf(fd, "%s [pid %-6ld] %s\n", log_date, (long)getpid(), message);
#endif /* LOG_DEBUG_INFORMATION */
	if(fd == stderr) fflush(fd);
	else fclose(fd);
	return (0);
}

