/*****************************************************************************
 *
 * db_macro_actions.c
 *
 * Copyright (c) 2004 Yves Mettier <ymettier@libertysurf.fr>
 * Last Modified:   2004-07-28
 * Few code lines from Ben Clewett from older perfparse/convert.c
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <time.h>
#include "string.h"
#include "common.h"
#include "dbms.h"
#include <config.h>
#include "libpp_common.h"
#include "db_macro_actions.h"
#include "del_policy.h"
#include <gettext.h>
#include <locale.h>

const char REG_VER_HOST[] = "dummy";
const char REG_VER_KEY[] = "pp/database/version";

void reset_host_delete_policy(const char*host_name) {

	GString *sql = g_string_new("");
	char    *h;
	int     iBinPolicy, iRawPolicy;

	iBinPolicy = getDefaultBinPolicy();
	iRawPolicy = getDefaultRawPolicy();

	h = strdup(sql_escape(host_name, 0));
	g_string_printf(sql, "SELECT host_name FROM perfdata_host WHERE host_name LIKE '%s'", h);
	query(sql->str);

	if (!rows()) {
		fprintf(stderr,"No host '%s' in database\n",host_name);
		g_string_free(sql,TRUE);
		exit(EXIT_FAILURE);
	}

	while ((result_row = mysql_fetch_row(query_result))) {

		printf("Setting host '%s' to default policy '%s' for Binary Data\n", sData(0), getPolicyDesc(iBinPolicy));
		printf("Setting host '%s' to default policy '%s' for Raw Data\n", sData(0), getPolicyDesc(iRawPolicy));

		g_string_printf(sql, "UPDATE perfdata_service_metric "
				"SET bin_delete_policy_type = 'host' ");
		g_string_append_printf(sql, ", bin_delete_policy = %s ", pval2sql(iBinPolicy));
		g_string_append_printf(sql, "WHERE host_name = '%s'", sData(0));
		query_no_return(sql->str);

		g_string_printf(sql, "UPDATE perfdata_service "
				"SET raw_delete_policy_type = 'host' ");
		g_string_append_printf(sql, ", raw_delete_policy = %s ", pval2sql(iRawPolicy));
		g_string_append_printf(sql, "WHERE host_name = '%s'", sData(0));
		query_no_return(sql->str);

		g_string_printf(sql, "UPDATE perfdata_host "
				"SET "
				"raw_delete_policy_type = 'template', "
				"bin_delete_policy_type = 'template', "
				"raw_delete_policy_name = '%s', "
				"bin_delete_policy_name = '%s', ",
				gettext(STRING_DELETE_POLICY_RAW_DEFAULT),
				gettext(STRING_DELETE_POLICY_BIN_DEFAULT));
		g_string_append_printf(sql, "bin_delete_policy = %s, ", pval2sql(iBinPolicy));
		g_string_append_printf(sql, "raw_delete_policy = %s ", pval2sql(iRawPolicy));
		g_string_append_printf(sql, "WHERE host_name = '%s'", sData(0));
		query_no_return(sql->str);

	}

	g_string_free(sql,TRUE);
}

/* Get the version from the database */
/* Version 0.101.1 onwards, from the registry. */
double getVersion()
{
	double version;
	char *s;

	setlocale(LC_NUMERIC, "POSIX");

	s = getRegistry(REG_VER_HOST, REG_VER_KEY, 0);

	if (s && s[0]) {
		version = atof(s);
	} else {
		version = DB_VERSION;
		setVersion(version);
	}

	setlocale(LC_NUMERIC, "");

	return version;

}

void setVersion(double version) 
{
	char s[10];

	setlocale(LC_NUMERIC, "POSIX"); 
	sprintf(s, "%.2f", version);
	setlocale(LC_NUMERIC, ""); 

	setRegistry(REG_VER_HOST, REG_VER_KEY, s, time(NULL));

}

int is_db_incorrect_version(void) {
	return (getVersion() < DB_VERSION);
}

void exit_if_db_version_not_correct(void) {
	if(is_db_incorrect_version()) {
		fprintf(stderr, "%s", db_version_error());
		exit(EXIT_FAILURE);
	}
}


char *db_version_error() {
#ifdef USE_GLIB12
	char*sstr;
#endif
	GString *s = NULL;

	if (s) return s->str;

	setlocale(LC_NUMERIC, "POSIX"); 

	s = g_string_new("Database schema version is not correct.\n");
	g_string_append_printf(s, "Current version %.2f.  Required version %.2f\n", getVersion(), DB_VERSION);
	g_string_append_printf(s, "Consider upgrading with perfparse-db-tool --update\n");

	setlocale(LC_NUMERIC, ""); 

#ifdef USE_GLIB12
	sstr = s->str;
	return(sstr);
#else
	return g_string_free(s,FALSE);
#endif

}




void check_db_integrity(void) {
	GString *str;
	str = g_string_new("");
	g_string_printf(str,"UPDATE perfdata_service_metric "
			"SET bin_delete_policy_name = '%s' "
			"WHERE bin_delete_policy_type = 'template' "
			"AND bin_delete_policy_name IS NULL",gettext(STRING_DELETE_POLICY_BIN_DEFAULT));
	query_no_return(str->str);

	g_string_printf(str,"UPDATE perfdata_service "
			"SET raw_delete_policy_name = '%s' "
			"WHERE raw_delete_policy_type = 'template' "
			"AND raw_delete_policy_name IS NULL",gettext(STRING_DELETE_POLICY_RAW_DEFAULT));
	query_no_return(str->str);

	g_string_printf(str,"UPDATE perfdata_host "
			"SET bin_delete_policy_name = '%s' "
			"WHERE bin_delete_policy_type = 'template' "
			"AND bin_delete_policy_name IS NULL",gettext(STRING_DELETE_POLICY_BIN_DEFAULT));
	query_no_return(str->str);

	g_string_printf(str,"UPDATE perfdata_host "
			"SET raw_delete_policy_name = '%s' "
			"WHERE raw_delete_policy_type = 'template' "
			"AND raw_delete_policy_name IS NULL",gettext(STRING_DELETE_POLICY_RAW_DEFAULT));
	query_no_return(str->str);
g_string_free(str,TRUE);
}

