/*****************************************************************************
 *
 * dbms.c - Common functions required by PerfParse suit.
 *
 * Copyright (c) 2004 Ben Clewett (perfparse@clewett.org.uk)
 * Last Modified:   2004-05-28
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <glib.h>
#include "libpp_common.h"
#include "common.h"
#include <netdb.h>
#include <dbms.h>
#include <config.h>
#include <gettext.h>

#undef SHOW_SQL
// #define SHOW_SQL

sql_failure_t sql_failure = NULL;
void freeResult(void);

/***************************************
 *
 * DBMS work
 *
 ***************************************/

/* DBMS identity: */
int		db_port = MYSQL_PORT;
MYSQL		db_mysql;
MYSQL_RES	*query_result = NULL;
MYSQL_ROW	result_row;

/* State Variables */
GString *s_SQL=NULL;
int 	iSQLCount = 0;
int	iRowsAffected;

int iData(int iCol)
{
	if (result_row[iCol] && result_row[iCol][0]) 
		return atoi(result_row[iCol]);
	else
		return 0;
}

int isNull(int iCol)
{
	if (result_row[iCol] == NULL) 
		return TRUE;
	return FALSE;
}

char *sData(int iCol)
{
	if (result_row[iCol] && result_row[iCol][0]) 
		return result_row[iCol];
	else
		return "";
}

void pp_mysql_default_sql_failure(char*f,int l, const char*str, const char *error) {
	pp_log(f,l,"%s (%s)",str,error);
	exit(EXIT_FAILURE);
}

void libpp_mysql_set_sql_failure_callback(sql_failure_t f) {
	sql_failure = f;
}

void libpp_mysql_query(char*file, int line, const char *sql)
{
	#ifdef SHOW_SQL
	printf("SQL:  \"%s\"\n", sql);
	fflush(stdout);
	#endif
	freeResult();

	if (mysql_query(&db_mysql, sql)) 
		sql_failure(file, line, sql, mysql_error(&db_mysql));

	iRowsAffected = mysql_affected_rows(&db_mysql);

	query_result = mysql_store_result(&db_mysql);

	#ifdef SHOW_SQL
	printf("Rows: %d\n", rows());
	fflush(stdout);
	#endif

	iSQLCount++;
}

int libpp_mysql_query_int(char*file, int line, const char *sql)
{
	int iRet;
	MYSQL_ROW	row;
	MYSQL_RES	*result;

	#ifdef SHOW_SQL
	printf("SQL:  \"%s\"\n", sql);
	fflush(stdout);
	#endif

	if (mysql_query(&db_mysql, sql)) 
		sql_failure(file, line, sql, mysql_error(&db_mysql));

	result = mysql_use_result(&db_mysql);
	row = mysql_fetch_row(result);

	if (row && row[0] && row[0][0]) 
		iRet = atoi(row[0]);
	else
		iRet = 0;

	if (result) mysql_free_result(result);

	iSQLCount++;

	return iRet;
}

const char *libpp_mysql_query_char(char*file, int line, const char *sql)
{
	static char *sRet = NULL;
	MYSQL_RES	*result;
	MYSQL_ROW	row;

	#ifdef SHOW_SQL
	printf("SQL:  \"%s\"\n", sql);
	fflush(stdout);
	 #endif

	if (sRet) free(sRet);
	
	if (mysql_query(&db_mysql, sql)) 
		sql_failure(file, line, sql, mysql_error(&db_mysql));

	result = mysql_use_result(&db_mysql);
	row = mysql_fetch_row(result);

	if (row && row[0] && row[0][0]) 
		sRet = strdup(row[0]);
	else
		sRet = NULL;

	if (result) mysql_free_result(result);

	iSQLCount++;

	return sRet;
}


void libpp_mysql_query_no_return(char*file, int line, const char *sql)
{
	MYSQL_RES	*discaud;

	#ifdef SHOW_SQL
	printf("SQL:  \"%s\"\n", sql);
	fflush(stdout);
	#endif

	if (mysql_query(&db_mysql, sql)) 
		sql_failure(file, line, sql, mysql_error(&db_mysql));

	iRowsAffected = mysql_affected_rows(&db_mysql);

	discaud = mysql_store_result(&db_mysql);
	if (discaud) {
		mysql_free_result(discaud);
	}


	iSQLCount++;


}





void freeResult(void)
{
	if (query_result) {
		mysql_free_result(query_result);
		query_result = NULL;
	}

}


int rows(void)
{
	if (query_result)
		return mysql_num_rows(query_result);
	else
		return 0;
}

int now_connect(int bTerminate)
{
	if(NULL == sql_failure) sql_failure = pp_mysql_default_sql_failure;
	if (!mysql_init (&db_mysql)) {
		pp_log(__FILE__,__LINE__, "Cannot initialise MySQL");
		sql_failure(__FILE__, __LINE__, "CONNECT", "Cannot initialise MySQL");
		exit(EXIT_FAILURE);
	}
	s_SQL = g_string_sized_new(SQL_SIZE);

	mysql_options (&db_mysql, MYSQL_READ_DEFAULT_GROUP, "mysqld");

	/* establish a connection to the MySQL server */
    if (!mysql_real_connect(&db_mysql, 
		config_get_value_as_string(CONFIG_ITEM_ID_DB_HOST), 
		config_get_value_as_string(CONFIG_ITEM_ID_DB_USER), 
		config_get_value_as_string(CONFIG_ITEM_ID_DB_PWD), 
		config_get_value_as_string(CONFIG_ITEM_ID_DB_DATABASE), 
		db_port, NULL, 0)) {

		g_string_printf(s_SQL, "Could not connect to MySQL using:  Database  = '%s',  Host  = '%s',  Username =  '%s', Password  = '%s'",
			config_get_value_as_string(CONFIG_ITEM_ID_DB_DATABASE), 
			config_get_value_as_string(CONFIG_ITEM_ID_DB_HOST), 
			config_get_value_as_string(CONFIG_ITEM_ID_DB_USER), 
			config_get_value_as_string(CONFIG_ITEM_ID_DB_PWD));

		mysql_close(&db_mysql);

		if (bTerminate)	{
			pp_log(__FILE__,__LINE__, "Error %s: %s", s_SQL->str, mysql_error(&db_mysql));
			sql_failure(__FILE__, __LINE__, s_SQL->str, mysql_error(&db_mysql));
			exit(EXIT_FAILURE);
		}
		else return FALSE;


	}

	return TRUE;

}

char* getRegistry(const char*host, const char*key, time_t *tm) {
	char *value=NULL;
	g_string_printf(s_SQL, "SELECT rvalue,ctime from perfdata_registry WHERE host = '%s' ", sql_escape(host,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, "AND rkey = '%s'", sql_escape(key,PERCENT_ESCAPE_CHAR));
	query(s_SQL->str);
	if ((result_row = mysql_fetch_row(query_result))) {
		value = strdup(sData(0));
		if(tm) tm[0] = iData(1);
		freeResult();
	}
	return(value);

}

void setRegistry(const char*host, const char*key, const char*value, time_t tm) {
	g_string_printf(s_SQL, "SELECT count(*) from perfdata_registry WHERE host = '%s' ", sql_escape(host,PERCENT_ESCAPE_CHAR));
	g_string_append_printf(s_SQL, "AND rkey = '%s'", sql_escape(key,PERCENT_ESCAPE_CHAR));
	query(s_SQL->str);
	result_row = mysql_fetch_row(query_result);
	if (iData(0) > 0) {
		freeResult();
		g_string_printf(s_SQL, "UPDATE perfdata_registry "
				"SET rvalue = '%s', ",sql_escape(value,PERCENT_ESCAPE_CHAR));

		g_string_append_printf(s_SQL, "ctime = FROM_UNIXTIME(%ld) ", tm);
		g_string_append_printf(s_SQL, "WHERE host = '%s' ", sql_escape(host,PERCENT_ESCAPE_CHAR));
		g_string_append_printf(s_SQL, "AND rkey = '%s'", sql_escape(key,PERCENT_ESCAPE_CHAR));

	} else {
		g_string_printf(s_SQL, "INSERT INTO perfdata_registry "
				"(host,rkey, rvalue,ctime) "
				"VALUES ('%s',",sql_escape(host,PERCENT_ESCAPE_CHAR));

		g_string_append_printf(s_SQL, "'%s', ", sql_escape(key,PERCENT_ESCAPE_CHAR));
		g_string_append_printf(s_SQL, "'%s', ", sql_escape(value,PERCENT_ESCAPE_CHAR));
		g_string_append_printf(s_SQL, "FROM_UNIXTIME(%ld)) ", tm);

	}
	query_no_return(s_SQL->str);	

}

int addField(char *sTable, char *sField, char *sDescript)
{

	/* See if the field is there. */
	g_string_printf(s_SQL, "EXPLAIN %s", sTable);
	query(s_SQL->str);
	while ((result_row = mysql_fetch_row(query_result))) {

		if (!strcmp(sData(0), sField)) return 0;
		
	}

	/* No field, add */
	g_string_printf(s_SQL, "ALTER TABLE %s ADD %s %s",
		sTable, sField, sDescript);
	query(s_SQL->str);

	return 1;


}

void dropField(char *sTable, char *sField)
{

	int bHit = 0;

	/* See if the field is there. */
	g_string_printf(s_SQL, "EXPLAIN %s", sTable);
	query(s_SQL->str);
	while ((result_row = mysql_fetch_row(query_result))) {

		if (!strcmp(sData(0), sField)) {
			bHit = 1;
			break;
		}
		
	}

	if (!bHit) return;

	/* Field, drop */
	g_string_printf(s_SQL, "ALTER TABLE %s DROP %s",
		sTable, sField);
	query(s_SQL->str);


}


char *sql_escape(const char *sIn,char percent_escape_char)
{
	static char sOut[1024];
	int i, j;

	if (!sIn) return "";

	j = 0;

	for (i = 0; sIn[i] && j < 1023; i++) {
		if (sIn[i] == '\'') sOut[j++] = '\'';
		else if((sIn[i] == '%') && percent_escape_char) sOut[j++] = percent_escape_char;
		else if(sIn[i] == '\\') sOut[j++] = '\\';

		sOut[j++] = sIn[i];

	}

	sOut[j] = 0;


	return sOut;


}

