/*****************************************************************************
 *
 * dbms.h - Common functions required by PerfParse suit.
 *
 * Copyright (c) Fabalabs Software GmbH, 2005
 *   oss@fabalabs.org
 *   Author: Friedrich Priewasser (friedrich.priewasser@fabalabs.org)
 *   Last Modified:   2005-01-13
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#ifndef PP_DBMS_H
#define PP_DBMS_H

#include <libpq-fe.h>
/* #include <pg_config.h> */ /* This conflicts with PP. */

#define PERCENT_ESCAPE_CHAR '\0'

/* DBMS identity: */
extern char*		db_port;
extern PGconn		*db_postgresql;
extern PGresult	*query_result;
extern int			currentRow;
#define SQL_SIZE 1024
extern GString		*s_SQL;
extern int			iSQLCount;
extern int			iRowsAffected;

int iData(int iCol);
int isNull(int iCol);
char *sData(int iCol);
int bData(int iCol);
time_t tData(int iCol);
time_t getTimeFromString(char *sYYYYMMDDhhmmss);
void writeDate(time_t t, char *s);
void libpp_postgresql_query(char*f,int l, const char *sql);
void libpp_postgresql_query_no_return(char*f,int l, const char *sql);
int  libpp_postgresql_query_int(char*f,int l, const char *sql);
const char *libpp_postgresql_query_char(char*f,int l, const char *sql);
#define query(x) libpp_postgresql_query(__FILE__,__LINE__,x)
#define query_no_return(x) libpp_postgresql_query_no_return(__FILE__,__LINE__,x)
#define query_int(x) libpp_postgresql_query_int(__FILE__,__LINE__,x)
#define query_char(x) libpp_postgresql_query_char(__FILE__,__LINE__,x)
int rows(void);
int now_connect(int bTerminate);
void setRegistry(const char*host, const char*key, const char*value, time_t tm);
char* getRegistry(const char*host, const char*key, time_t *tm);
int addField(char *sTable, char *sField, char *sDescript);
void dropField(char *sTable, char *sField);
char *sql_escape(const char *sIn,char percent_escape_char);

/* Prototype for any sql_fail() function that has to be defined somewhere else but not in dbms.c */
typedef void (*sql_failure_t)(char*f,int l, const char *sql, const char *error);
void libpp_postgresql_set_sql_failure_callback(sql_failure_t f);

#endif
