/*****************************************************************************
 *
 * storage.h - Parse Nagios data and store in relational database.
 *
 * Copyright (c) 2004 Yves Mettier (ymettier@libertysurf.fr)
 * Last Modified:   2004-09-01
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#ifndef __LIB_STORAGE__
#define __LIB_STORAGE__

#include <time.h>
#include <stdio.h>
#include <libnagios_perfdata_parser.h>

typedef struct storage_module_t {
	char id[30];
	char version[10];
	void *params;
	int enabled;
	int (*set_registry)(struct storage_module_t *sm, const char*hostname, const char*key, const char*value, time_t tm);
	int (*store_line)(struct storage_module_t *sm, const log_line_t *log_data);
	int (*store_perf_values)(struct storage_module_t *sm, const log_line_t *log_data, const perf_fields_t *perf_fields, const perf_value_t *perf_value);
	int (*connect)(struct storage_module_t *sm);
	int (*disconnect)(struct storage_module_t *sm);
	void (*atexit)(struct storage_module_t *sm);
	void (*print_statistics)(struct storage_module_t *sm, FILE*fd);
	void (*print_usage)(struct storage_module_t *sm, FILE*fd, const char*id);
	void (*periodic_cleanup)(struct storage_module_t *sm);

	
	int (*log)(char *file, int line, char *fmt, ...);
} storage_module_t;

storage_module_t *storage_new(const char*id, const char*version);

#endif
