/*****************************************************************************
 *
 * storage_print.c - Parse Nagios data and store in relational database.
 *
 * Copyright (c) 2004 Yves Mettier (ymettier@libertysurf.fr)
 * Last Modified:   2004-09-01
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#define STORAGE_PRINT_VERSION "0.0.1"

#include <stdio.h>
#include <stdlib.h>
#include <libnagios_perfdata_parser.h>
#include "libpp_storage.h"

#include <config.h>
#include <gettext.h>
#include <locale.h>

int storage_print_store_line(storage_module_t *this, const log_line_t *log_data) {
	printf("PM(store_line) TIMET='%s'\n", log_data->macro[LOG_LINE_TIMET]);
	printf("PM(store_line) HOSTNAME='%s'\n", log_data->macro[LOG_LINE_HOSTNAME]);
	printf("PM(store_line) SERVICEDESC='%s'\n", log_data->macro[LOG_LINE_SERVICEDESC]);
	printf("PM(store_line) OUTPUT='%s'\n", log_data->macro[LOG_LINE_OUTPUT]);
	printf("PM(store_line) SERVICESTATE='%s'\n", log_data->macro[LOG_LINE_SERVICESTATE]);
	return(0);
}

int storage_print_store_perf_values(storage_module_t *this, const log_line_t *log_data, const perf_fields_t *perf_fields, const perf_value_t *perf_value) {

	setlocale(LC_NUMERIC, "POSIX"); 

	if((NULL != perf_value) && (perf_fields->type == PERF_FIELDS_TYPE_PERF)) {
		printf("PM(store_perf_values) label='%s' value='%s' type=%s\n", perf_fields->label,perf_fields->value,(perf_fields->type == PERF_FIELDS_TYPE_PERF)?"perf":"string");
		printf("PM(store_perf_values) %s => {\n",perf_fields->label);
		printf("PM(store_perf_values) \t\thostname='%s',\n",log_data->macro[LOG_LINE_HOSTNAME]);
		printf("PM(store_perf_values) \t\tservice='%s',\n",log_data->macro[LOG_LINE_SERVICEDESC]);
		printf("PM(store_perf_values) \t\tmetric='%s',\n",perf_fields->label);
		printf("PM(store_perf_values) \t\tvalue=%.14e,\n",perf_value->d[PERF_VALUE_VALUE]);
		printf("PM(store_perf_values) \t\tunit='%s',\n",perf_value->unit);
		printf("PM(store_perf_values) \t\twarn_start=%.14e,\n",perf_value->d[PERF_VALUE_WARN_START]);
		printf("PM(store_perf_values) \t\twarn_end=%.14e,\n",perf_value->d[PERF_VALUE_WARN_END]);
		printf("PM(store_perf_values) \t\twarn_inverted=%s,\n",(perf_value->ranges_type & PERF_VALUE_RANGE_WARNING_RANGE_INSIDE)?"yes":"no");
		printf("PM(store_perf_values) \t\tcrit_start=%.14e,\n",perf_value->d[PERF_VALUE_CRIT_START]);
		printf("PM(store_perf_values) \t\tcrit_end=%.14e,\n",perf_value->d[PERF_VALUE_CRIT_END]);
		printf("PM(store_perf_values) \t\tcrit_inverted=%s,\n",(perf_value->ranges_type & PERF_VALUE_RANGE_CRITICAL_RANGE_INSIDE)?"yes":"no");
		printf("PM(store_perf_values) \t\tmetric_state=%d,\n",perf_value->metric_state);
		printf("PM(store_perf_values) \t\t\t}\n");
	}

	setlocale(LC_NUMERIC, ""); 

	return(0);
}

int storage_print_set_registry(storage_module_t *this, const char*hostname, const char*key, const char*value, time_t tm) {
	printf("PM(set_registry) %ld '%s/%s' = '%s'\n", tm,hostname,key,value);
	return(0);
}

void storage_print_atexit(struct storage_module_t *sm) {
	printf("Leaving...\n");
}

void storage_print_print_statistics(storage_module_t *this, FILE*fd) {
	fprintf(fd, "No statistics for print storage module\n");
}

storage_module_t* storage_module_init(void) {
	storage_module_t *sm;
	sm = storage_new("print", STORAGE_PRINT_VERSION);
	if(NULL == sm) return(NULL);
	sm->store_line = storage_print_store_line;
	sm->set_registry = storage_print_set_registry;
	sm->store_perf_values = storage_print_store_perf_values;
	sm->atexit = storage_print_atexit;
	sm->print_statistics = storage_print_print_statistics;
	sm->enabled = 1;
	return(sm);
}

