/*****************************************************************************
 *
 * log_reader.h - Parse Nagios data and store in relational database.
 *
 * Copyright (c) 2004 Yves Mettier (ymettier@libertysurf.fr)
 * Last Modified:   2004-08-25
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#ifndef __LOG_READER__
#define __LOG_READER__

#include <glib.h>

typedef enum { 
	LOG_FD_FILE, 
	LOG_FD_STDIN, 
	LOG_FD_PIPE, 
	LOG_FD_SOCKET_MAIN, 
	LOG_FD_SOCKET, 
	LOG_FD_CLIENT_SOCKET, 
	LOG_FD_UNDEF,
	LOG_FD_NOEXIST
} log_fd_type_e;

typedef struct {
	GString*buffer;
	guint file_pos;
	guint file_last_line_hash;
	char *file_name;
	int fd;
	log_fd_type_e type;
	short ready;
} log_fd_t;


int log_source_type(int i);
int log_source_output_write(int i, const char*str1, const char*str2);
int log_source_input_is_ready(int i);
void log_source_test_input(void);
int open_log_source(char*filename);
void log_source_command(int id);
int log_source_new_server(int port);
int log_source_new_connected_client(int socket_id);
int close_log_source(int i, int shutd);
void close_all_log_sources(void);
GString*log_reader(int i, GString*buffer);

void log_reader_install_signal_handler(void);
void log_reader_remove_signal_handler(void);
void log_reader_mask_signals(void);
void log_reader_unmask_signals(void);

#endif
